@@ -1,22 +1,25 @@
 import requests
 import json
+import csv
 import geojson
-
+import logging
+from io import StringIO
 from .errors import (OverpassSyntaxError, TimeoutError, MultipleRequestsError,
                      ServerLoadError, UnknownOverpassError, ServerRuntimeError)
 
-class API(object):
-    """A simple Python wrapper for the OpenStreetMap Overpass API"""
 
-    SUPPORTED_FORMATS = ["geojson", "json", "xml"]
+class API(object):
+    """A simple Python wrapper for the OpenStreetMap Overpass API."""
+
+    SUPPORTED_FORMATS = ["geojson", "json", "xml", "csv"]
 
     # defaults for the API class
-    _timeout = 25  # seconds
+    _timeout = 25  # second
     _endpoint = "https://overpass-api.de/api/interpreter"
     _debug = False
 
     _QUERY_TEMPLATE = "[out:{out}];{query}out {verbosity};"
-    _GEOJSON_QUERY_TEMPLATE = "[out:json];{query}out body geom;"
+    _GEOJSON_QUERY_TEMPLATE = "[out:json];{query}out {verbosity};"
 
     def __init__(self, *args, **kwargs):
         self.endpoint = kwargs.get("endpoint", self._endpoint)
@@ -25,36 +28,62 @@ class API(object):
         self._status = None
 
         if self.debug:
-            import httplib
-            import logging
-            httplib.HTTPConnection.debuglevel = 1
+            # https://stackoverflow.com/a/16630836
+            try:
+                import http.client as http_client
+            except ImportError:
+                # Python 2
+                import httplib as http_client
+            http_client.HTTPConnection.debuglevel = 1
 
+            # You must initialize logging,
+            # otherwise you'll not see debug output.
             logging.basicConfig()
             logging.getLogger().setLevel(logging.DEBUG)
             requests_log = logging.getLogger("requests.packages.urllib3")
             requests_log.setLevel(logging.DEBUG)
             requests_log.propagate = True
 
-    def Get(self, query, responseformat="geojson", verbosity="body", build=True):
-        """Pass in an Overpass query in Overpass QL"""
-
+    def get(self,
+            query,
+            responseformat="geojson",
+            verbosity="body",
+            build=True):
+        """Pass in an Overpass query in Overpass QL."""
         # Construct full Overpass query
         if build:
-            full_query = self._ConstructQLQuery(query, responseformat=responseformat, verbosity=verbosity)
+            full_query = self._construct_ql_query(
+                query,
+                responseformat=responseformat,
+                verbosity=verbosity)
         else:
             full_query = query
-        
+
+        if self.debug:
+            logging.getLogger().info(query)
+
         # Get the response from Overpass
-        raw_response = self._GetFromOverpass(full_query)
+        r = self._get_from_overpass(full_query)
+        content_type = r.headers.get('content-type')
 
-        if responseformat == "xml" or responseformat.startswith("csv"):
-            return raw_response
-            
-        response = json.loads(raw_response)
+        if self.debug:
+            print(content_type)
+        if content_type == "text/csv":
+            result = []
+            reader = csv.reader(StringIO(r.text), delimiter='\t')
+            for row in reader:
+                result.append(row)
+            return result
+        elif content_type == "text/xml" or content_type == "application/xml":
+            return r.text
 
-        # Check for valid answer from Overpass. A valid answer contains an 'elements' key at the root level.
+        response = json.loads(r.text)
+
+        # Check for valid answer from Overpass.
+        # A valid answer contains an 'elements' key at the root level.
         if "elements" not in response:
-            raise UnknownOverpassError("Received an invalid answer from Overpass.")
+            raise UnknownOverpassError(
+                "Received an invalid answer from Overpass.")
 
         # If there is a 'remark' key, it spells trouble.
         overpass_remark = response.get('remark', None)
@@ -65,32 +94,38 @@ class API(object):
             return response
 
         # construct geojson
-        return self._asGeoJSON(response["elements"])
+        return self._as_geojson(response["elements"])
 
-    def Search(self, feature_type, regex=False):
+    def search(self, feature_type, regex=False):
         """Search for something."""
         raise NotImplementedError()
 
-    def _ConstructQLQuery(self, userquery, responseformat, verbosity):
+    # deprecation of upper case functions
+    Get = get
+    Search = search
+
+    def _construct_ql_query(self, userquery, responseformat, verbosity):
         raw_query = str(userquery)
         if not raw_query.endswith(";"):
             raw_query += ";"
 
         if responseformat == "geojson":
             template = self._GEOJSON_QUERY_TEMPLATE
-            complete_query = template.format(query=raw_query, verbosity=verbosity)
+            complete_query = template.format(
+                query=raw_query,
+                verbosity=verbosity)
         else:
             template = self._QUERY_TEMPLATE
-            complete_query = template.format(query=raw_query, out=responseformat, verbosity=verbosity)
+            complete_query = template.format(
+                query=raw_query,
+                out=responseformat,
+                verbosity=verbosity)
 
         if self.debug:
             print(complete_query)
         return complete_query
 
-    def _GetFromOverpass(self, query):
-        """This sends the API request to the Overpass instance and
-        returns the raw result, or an error."""
-
+    def _get_from_overpass(self, query):
         payload = {"data": query}
 
         try:
@@ -100,7 +135,7 @@ class API(object):
                 timeout=self.timeout,
                 headers={'Accept-Charset': 'utf-8;q=0.7,*;q=0.7'}
             )
-            
+
         except requests.exceptions.Timeout:
             raise TimeoutError(self._timeout)
 
@@ -115,25 +150,25 @@ class API(object):
                 raise ServerLoadError(self._timeout)
             raise UnknownOverpassError(
                 "The request returned status code {code}".format(
-                    code=self._status
-                    )
-                )
+                    code=self._status))
         else:
             r.encoding = 'utf-8'
-            return r.text
+            return r
 
-    def _asGeoJSON(self, elements):
+    def _as_geojson(self, elements):
 
         features = []
         for elem in elements:
-            elem_type = elem["type"]
-            if elem_type == "node":
-                geometry = geojson.Point((elem["lon"], elem["lat"]))
-            elif elem_type == "way":
+            elem_type = elem.get("type")
+            if elem_type and elem_type == "node":
+                geometry = geojson.Point((elem.get("lon"), elem.get("lat")))
+            elif elem_type and elem_type == "way":
                 points = []
-                for coords in elem["geometry"]:
-                    points.append((coords["lon"], coords["lat"]))
-                geometry = geojson.LineString(points)
+                geom = elem.get("geometry")
+                if geom:
+                    for coords in elem.get("geometry"):
+                        points.append((coords["lon"], coords["lat"]))
+                    geometry = geojson.LineString(points)
             else:
                 continue
 
