@@ -4,11 +4,12 @@
 class MapQuery(object):
     """Query to retrieve complete ways and relations in an area."""
 
-    _QUERY_TEMPLATE = "(node({south},{west},{north},{east});<;);"
+    _QUERY_TEMPLATE = "(node({south},{west},{north},{east});<;>;);"
 
     def __init__(self, south, west, north, east):
         """
         Initialize query with given bounding box.
+
         :param bbox Bounding box with limit values in format west, south,
         east, north.
         """
@@ -27,15 +28,15 @@ class MapQuery(object):
 
 
 class WayQuery(object):
-    """Query to retrieve a set of ways and their dependent nodes satisfying
-    the input parameters"""
+    """Query to retrieve a set of ways and their dependent nodes satisfying the input parameters."""
 
     _QUERY_TEMPLATE = "(way{query_parameters});(._;>;);"
 
     def __init__(self, query_parameters):
         """Initialize a query for a set of ways satisfying the given parameters.
-        :param query_parameters Overpass QL query parameters"""
 
+        :param query_parameters Overpass QL query parameters
+        """
         self.query_parameters = query_parameters
 
     def __str__(self):
